@@ -2,32 +2,34 @@ import os
 import sys
 import shutil
 import pkgutil
-import requests
 import subprocess
 import threading
 
 from PySide6.QtCore import Qt, QCoreApplication, QObject, QThread, QWaitCondition, QMutex, QDataStream
 from PySide6.QtCore import QByteArray, QEvent, Signal, Slot, QTranslator, QLocale, QLibraryInfo
 from PySide6.QtGui import QIcon, QKeyEvent, QKeySequence, QShortcut
-from PySide6.QtWidgets import QApplication, QDialog, QMessageBox, QLabel, QPushButton, QCheckBox, QProgressBar, QVBoxLayout
+from PySide6.QtWidgets import QApplication, QDialog, QMessageBox, QLabel, QPushButton, QCheckBox
+from PySide6.QtWidgets import QProgressBar, QVBoxLayout, QSpacerItem, QSizePolicy
 from PySide6.QtUiTools import QUiLoader
+from PySide6.QtDBus import QDBusConnection
 
-from pupgui2.resources import ui
-from pupgui2.constants import APP_NAME, APP_VERSION, BUILD_INFO, TEMP_DIR, STEAM_STL_INSTALL_PATH
-from pupgui2.constants import STEAM_PROTONGE_FLATPAK_APPSTREAM, STEAM_BOXTRON_FLATPAK_APPSTREAM, STEAM_STL_FLATPAK_APPSTREAM
+from pupgui2.constants import APP_NAME, APP_VERSION, APP_ID, BUILD_INFO, TEMP_DIR, STEAM_STL_INSTALL_PATH
+from pupgui2.constants import STEAM_BOXTRON_FLATPAK_APPSTREAM, STEAM_STL_FLATPAK_APPSTREAM, IS_FLATPAK
 from pupgui2 import ctloader
 from pupgui2.datastructures import CTType, MsgBoxType, MsgBoxResult
 from pupgui2.gamepadinputworker import GamepadInputWorker
 from pupgui2.pupgui2aboutdialog import PupguiAboutDialog
 from pupgui2.pupgui2ctinfodialog import PupguiCtInfoDialog
 from pupgui2.pupgui2customiddialog import PupguiCustomInstallDirectoryDialog
+from pupgui2.pupgui2exceptionhandler import PupguiExceptionHandler
 from pupgui2.pupgui2gamelistdialog import PupguiGameListDialog
 from pupgui2.pupgui2installdialog import PupguiInstallDialog
-from pupgui2.steamutil import get_steam_acruntime_list, get_steam_app_list, get_steam_ct_game_map
+from pupgui2.steamutil import get_steam_acruntime_list, get_steam_app_list, get_steam_ct_game_map, get_steam_global_ctool_name, ctool_is_runtime_for_app
 from pupgui2.heroicutil import is_heroic_launcher, get_heroic_game_list
+from pupgui2.dbusutil import dbus_progress_message
 from pupgui2.util import apply_dark_theme, create_compatibilitytools_folder, get_installed_ctools, remove_ctool
 from pupgui2.util import install_directory, available_install_directories, get_install_location_from_directory_name
-from pupgui2.util import print_system_information, single_instance, download_awacy_gamelist, is_online, config_advanced_mode, compat_tool_available
+from pupgui2.util import print_system_information, single_instance, download_awacy_gamelist, is_online, config_advanced_mode, config_github_access_token, config_gitlab_access_token, compat_tool_available
 
 
 class InstallWineThread(QThread):
@@ -80,10 +82,13 @@ class MainWindow(QObject):
     def __init__(self):
         super(MainWindow, self).__init__()
 
-        self.rs = requests.Session()
-        if token := os.getenv('PUPGUI_GHA_TOKEN'):
-            self.rs.headers.update({'Authorization': f'token {token}'})
+        self.web_access_tokens: dict[str, str] = {
+            'github': os.getenv('PUPGUI_GHA_TOKEN') or config_github_access_token(),
+            'gitlab': os.getenv('PUPGUI_GLA_TOKEN') or config_gitlab_access_token(),
+        }
+
         self.ct_loader = ctloader.CtLoader(main_window=self)
+        _ = self.ct_loader.load_ctmods()
 
         for ctobj in self.ct_loader.get_ctobjs():
             cti = ctobj.get('installer')
@@ -101,6 +106,9 @@ class MainWindow(QObject):
         self.msgcb_answer : MsgBoxResult = None
         self.msgcb_answer_lock = QMutex()
 
+        self.dbus_session_bus = QDBusConnection.sessionBus()
+        _ = dbus_progress_message(-1, 0)  # Reset any previously set download information to be blank
+
         self.load_ui()
         self.setup_ui()
         self.update_statusbar_message.connect(self.ui.statusBar().showMessage)
@@ -121,7 +129,7 @@ class MainWindow(QObject):
         self.progressBarDownload = QProgressBar()
         self.progressBarDownload.setVisible(False)
         self.ui.statusBar().addPermanentWidget(self.progressBarDownload)
-        self.ui.setWindowIcon(QIcon.fromTheme('net.davidotek.pupgui2'))
+        self.ui.setWindowIcon(QIcon.fromTheme(APP_ID))
         self.ui.txtInstalledVersions.setText('0')
 
         self.update_combo_install_location()
@@ -135,6 +143,7 @@ class MainWindow(QObject):
         self.ui.btnClose.clicked.connect(self.btn_close_clicked)
         self.ui.listInstalledVersions.itemDoubleClicked.connect(self.list_installed_versions_item_double_clicked)
         self.ui.listInstalledVersions.itemSelectionChanged.connect(self.list_installed_versions_item_selection_changed)
+        self.ui.listInstalledVersions.setStyleSheet('QListWidget::item { padding: 3px; }')
         self.ui.btnShowCtInfo.clicked.connect(self.btn_show_ct_info_clicked)
         self.ui.btnSteamFlatpakCtools.clicked.connect(self.btn_steam_flatpak_ctools_clicked)
 
@@ -183,13 +192,26 @@ class MainWindow(QObject):
         t = threading.Thread(target=_set_default_statusbar_thread, args=[self.update_statusbar_message])
         t.start()
 
-    def update_combo_install_location(self):
+    def send_dbus_download_progress(self, progress: float) -> None:
+
+        """
+        Send Download Progress and Pending Downloads count using DBus.
+        """
+
+        progress_pct = progress / 100  # DBus progress signal expects progress between 0-1
+        num_downloads = len(self.pending_downloads)
+        if progress < 0:  # negative progress indicates cancellation/failure/etc
+            num_downloads = 0
+
+        _ = dbus_progress_message(progress_pct, num_downloads, self.dbus_session_bus)
+
+    def update_combo_install_location(self, custom_install_dir = None):
         self.updating_combo_install_location = True
 
         self.ui.comboInstallLocation.clear()
         self.combo_install_location_index_map = []
 
-        current_install_dir = install_directory()
+        current_install_dir = custom_install_dir or install_directory()
         for i, install_dir in enumerate(available_install_directories()):
             icon_name = get_install_location_from_directory_name(install_dir).get('icon')
             display_name = get_install_location_from_directory_name(install_dir).get('display_name')
@@ -199,9 +221,14 @@ class MainWindow(QObject):
                 self.ui.comboInstallLocation.addItem(install_dir)
             self.combo_install_location_index_map.append(install_dir)
             if current_install_dir == install_dir:
+                if custom_install_dir is not None:
+                    self.updating_combo_install_location = False
                 self.ui.comboInstallLocation.setCurrentIndex(i)
 
         self.updating_combo_install_location = False
+        # Update compat list when custom install directoy is removed -- Not called because of `self.updating_combo_install_location = False` -- Could be improved?
+        if custom_install_dir is not None and len(custom_install_dir) <= 0:
+            self.ui.comboInstallLocation.currentIndexChanged.emit(self.ui.comboInstallLocation.currentIndex())
 
     def update_ui(self):
         """ update ui contents """
@@ -220,20 +247,34 @@ class MainWindow(QObject):
             self.get_installed_versions('vkd3d', vkd3d_dir)
         # Launcher specific (Steam): Number of games using the compatibility tool
         elif install_loc.get('launcher') == 'steam' and 'vdf_dir' in install_loc:
-            get_steam_app_list(install_loc.get('vdf_dir'), cached=False)  # update app list cache
+            steam_app_list = get_steam_app_list(install_loc.get('vdf_dir'), cached=False)  # update app list cache
+            global_ctool_name: str = get_steam_global_ctool_name(install_loc.get('vdf_dir'))
             self.compat_tool_index_map += get_steam_acruntime_list(install_loc.get('vdf_dir'), cached=True)
-            map = get_steam_ct_game_map(install_loc.get('vdf_dir'), self.compat_tool_index_map, cached=True)
+            ct_game_map = get_steam_ct_game_map(install_loc.get('vdf_dir'), self.compat_tool_index_map, cached=True)
             for ct in self.compat_tool_index_map:
-                ct.no_games = len(map.get(ct, []))
+                ct.no_games = len(ct_game_map.get(ct, []))
+                ct_name = ct.get_internal_name()
+                if ct_name == global_ctool_name:
+                    ct.set_global()  # Set (global) text
+                    self.compat_tool_index_map.insert(0, self.compat_tool_index_map.pop(self.compat_tool_index_map.index(ct)))  # Move global ctool to top of list
+                # Runtime length has to be calculated separately as they are not stored in the compat_tool_index_map, runtimes are dependencies of apps and not selected compatibility tools
+                if ct.ct_type == CTType.STEAM_RT:
+                    ct.no_games += len([game for game in steam_app_list if ctool_is_runtime_for_app(game, ct)])
         # Launcher specific (Heroic): Set number of installed games using compat tool
         elif is_heroic_launcher(install_loc.get('launcher')):
             heroic_dir = os.path.join(os.path.expanduser(install_loc.get('install_dir')), '../..')
             heroic_game_list = get_heroic_game_list(heroic_dir)
             for ct in self.compat_tool_index_map:
                 ct.no_games = len([game for game in heroic_game_list if game.is_installed and ct.displayname in game.wine_info.get('name', '')])
+            
+            # Get DXVK/VKD3D installs for Heroic
+            dxvk_dir = os.path.join(install_directory(), '../dxvk')
+            vkd3d_dir = os.path.join(install_directory(), '../vkd3d')
+            self.get_installed_versions('dxvk', dxvk_dir)
+            self.get_installed_versions('vkd3d', vkd3d_dir)
 
         for ct in self.compat_tool_index_map:
-            self.ui.listInstalledVersions.addItem(ct.get_displayname(unused_tr=self.tr('unused')))
+            self.ui.listInstalledVersions.addItem(ct.get_displayname(unused_tr=self.tr('unused'), global_tr=self.tr('global')))
             if ct.no_games == 0:
                 unused_ctools += 1
 
@@ -257,6 +298,10 @@ class MainWindow(QObject):
         self.ui.txtUnusedVersions.setText(self.tr('Unused: {unused_ctools}').format(unused_ctools=unused_ctools) if unused_ctools > 0 else '')
         self.ui.txtInstalledVersions.setText(f'{len(self.compat_tool_index_map)}')
 
+        combo_install_location_val: str = self.ui.comboInstallLocation.currentText()
+        if len(combo_install_location_val) > 0:
+            self.ui.comboInstallLocation.setToolTip(combo_install_location_val)
+
     def get_installed_versions(self, ctool_name, ctool_dir):
         for ct in get_installed_ctools(ctool_dir):
             if ctool_name not in ct.get_displayname().lower():
@@ -287,14 +332,12 @@ class MainWindow(QObject):
         if len(self.pending_downloads):
             compat_tool = self.pending_downloads[0]
             self.current_compat_tool_name = compat_tool['name'] + ' ' + compat_tool['version']
-        if value == -2:
+        elif value == -2:
             self.ui.statusBar().showMessage(self.tr('Download canceled.'))
             self.progressBarDownload.setVisible(False)
-            return
-        if value == -1:
+        elif value == -1:
             self.ui.statusBar().showMessage(self.tr('Could not install {current_compat_tool_name}...').format(current_compat_tool_name=self.current_compat_tool_name))
             self.progressBarDownload.setVisible(False)
-            return
         if value == 1:
             self.progressBarDownload.setVisible(True)
             self.ui.comboInstallLocation.setEnabled(False)
@@ -308,6 +351,9 @@ class MainWindow(QObject):
             self.ui.statusBar().showMessage(self.tr('Installed {current_compat_tool_name}.').format(current_compat_tool_name=self.current_compat_tool_name))
             self.update_ui()
 
+        # Send DBus progress
+        self.send_dbus_download_progress(value)
+
     def btn_add_version_clicked(self, compat_tool: str = ''):
         advanced_mode = (config_advanced_mode() == 'enabled')
         install_loc = get_install_location_from_directory_name(install_directory())
@@ -354,24 +400,27 @@ class MainWindow(QObject):
                 self.cancel_download(cancel_all=True)
                 self.ui.close()
 
-    def combo_install_location_current_index_changed(self):
+    def combo_install_location_current_index_changed(self) -> None:
+        if len(self.combo_install_location_index_map) <= 0:
+            self.update_ui()
+
+            return
+
         if not self.updating_combo_install_location:
             install_dir = install_directory(self.combo_install_location_index_map[self.ui.comboInstallLocation.currentIndex()])
             self.ui.statusBar().showMessage(self.tr('Changed install directory to {install_dir}.').format(install_dir=install_dir), timeout=3000)
-            self.update_ui()
+
+        self.update_ui()
 
     def btn_manage_install_locations_clicked(self):
-        customid_dialog = PupguiCustomInstallDirectoryDialog(parent=self.ui)
+        customid_dialog = PupguiCustomInstallDirectoryDialog(install_directory(), parent=self.ui)
         customid_dialog.custom_id_set.connect(self.update_combo_install_location)
 
     def show_launcher_specific_information(self):
         install_loc = get_install_location_from_directory_name(install_directory())
-        # For Steam Flatpak only: Show that GE-Proton and Boxtron are available directly from Flathub.
-        if 'steam' in install_loc.get('launcher', '') and 'Flatpak' in install_loc.get('display_name', ''):
-            self.ui.statusBar().showMessage(self.tr('Info: You can get GE-Proton / Boxtron directly from Flathub!'))
-            self.ui.btnSteamFlatpakCtools.setVisible(True)
-        else:
-            self.ui.btnSteamFlatpakCtools.setVisible(False)
+        self.ui.btnSteamFlatpakCtools.setVisible(
+            'steam' in install_loc.get('launcher', '') and 'Flatpak' in install_loc.get('display_name', '')
+            )
     
     def list_installed_versions_item_double_clicked(self, item):
         """ Show info about compatibility tool when double clicked in list """
@@ -394,6 +443,9 @@ class MainWindow(QObject):
             if ct.ct_type in [CTType.STEAM_CT, CTType.STEAM_RT]:
                 self.ui.btnRemoveSelected.setEnabled(False)
                 break
+            if ct.is_global:
+                self.ui.btnRemoveSelected.setEnabled(False)
+                break
 
     def btn_show_ct_info_clicked(self):
         install_loc = get_install_location_from_directory_name(install_directory())
@@ -403,22 +455,19 @@ class MainWindow(QObject):
             cti_dialog.batch_update_complete.connect(self.update_ui)
 
     def btn_steam_flatpak_ctools_clicked(self):
-        """ Open dialog to open the appstore(appstream) to install Proton-GE/Boxtron from Flathub"""
+        """ Open dialog to open the appstore(appstream) to install Boxtron from Flathub"""
         iftdialog = QDialog(parent=self.ui)
         iftdialog.setWindowTitle(self.tr('Install tool from Flathub'))
-        iftdialog.setFixedSize(250, 100)
         iftdialog.setModal(True)
         lbl_description = QLabel(self.tr('Click to open your app store'))
-        btn_dl_protonge = QPushButton('Proton-GE')
         btn_dl_boxtron = QPushButton('Boxtron')
         btn_dl_stl = QPushButton('Steam Tinker Launch')
         layout1 = QVBoxLayout()
         layout1.addWidget(lbl_description)
-        layout1.addWidget(btn_dl_protonge)
+        layout1.addSpacerItem(QSpacerItem(0, 0, QSizePolicy.Expanding, QSizePolicy.Expanding))
         layout1.addWidget(btn_dl_boxtron)
         layout1.addWidget(btn_dl_stl)
         iftdialog.setLayout(layout1)
-        btn_dl_protonge.clicked.connect(lambda: os.system(f'xdg-open {STEAM_PROTONGE_FLATPAK_APPSTREAM}'))
         btn_dl_boxtron.clicked.connect(lambda: os.system(f'xdg-open {STEAM_BOXTRON_FLATPAK_APPSTREAM}'))
         btn_dl_stl.clicked.connect(lambda: os.system(f'xdg-open {STEAM_STL_FLATPAK_APPSTREAM}'))
         iftdialog.show()
@@ -512,8 +561,10 @@ def main():
     app = PupguiApp(sys.argv)
     app.setApplicationName(APP_NAME)
     app.setApplicationVersion(APP_VERSION)
-    app.setWindowIcon(QIcon.fromTheme('net.davidotek.pupgui2'))
-    app.setDesktopFileName('net.davidotek.pupgui2.desktop')
+    app.setWindowIcon(QIcon.fromTheme(APP_ID))
+    app.setDesktopFileName(APP_ID)
+
+    PupguiExceptionHandler(app)
 
     lang = QLocale.languageToCode(QLocale().language())
     lname = QLocale().name()
@@ -525,7 +576,7 @@ def main():
         ldata = pkgutil.get_data(__name__, f'resources/i18n/pupgui2_{lname}.qm')  # Example: pupgui2_zh_TW.qm
     except:
         pass
-    finally:
+    else:
         translator = QTranslator()
         if translator.load(ldata):
             app.installTranslator(translator)
@@ -535,7 +586,7 @@ def main():
             ldata = pkgutil.get_data(__name__, f'resources/i18n/pupgui2_{lang}.qm') # Example: pupgui2_de.qm
         except:
             pass
-        finally:
+        else:
             translator = QTranslator()
             if translator.load(ldata):
                 app.installTranslator(translator)
@@ -552,7 +603,8 @@ def main():
     shutil.rmtree(TEMP_DIR, ignore_errors=True)
 
     # Flatpak workaround: Delete STL dir if it isn't installed (folder is always created for sandbox access)
-    if os.path.exists('/.flatpak-info') and len(os.listdir(STEAM_STL_INSTALL_PATH)) == 0:
+    if IS_FLATPAK and len(os.listdir(STEAM_STL_INSTALL_PATH)) == 0:
         subprocess.run(['flatpak-spawn', '--host', 'rm', '-r', STEAM_STL_INSTALL_PATH])
 
+    _ = dbus_progress_message(-1, 0)  # Reset any previously set download information to be blank
     sys.exit(ret)
