@@ -1,5 +1,5 @@
 import os
-from typing import Dict, List
+from typing import Dict, List, Union
 import shutil
 import subprocess
 import json
@@ -7,21 +7,41 @@ import vdf
 import requests
 import threading
 import pkgutil
+import binascii
 from steam.utils.appcache import parse_appinfo
 
 from PySide6.QtCore import Signal
 from PySide6.QtWidgets import QMessageBox, QApplication
 
 from pupgui2.constants import APP_NAME, APP_ID, APP_ICON_FILE
+from pupgui2.constants import PROTON_EAC_RUNTIME_APPID, PROTON_BATTLEYE_RUNTIME_APPID, PROTON_NEXT_APPID, STEAMLINUXRUNTIME_APPID, STEAMLINUXRUNTIME_SOLDIER_APPID, STEAMLINUXRUNTIME_SNIPER_APPID
 from pupgui2.constants import LOCAL_AWACY_GAME_LIST, PROTONDB_API_URL
-from pupgui2.constants import STEAM_STL_INSTALL_PATH, STEAM_STL_CONFIG_PATH, STEAM_STL_SHELL_FILES, STEAM_STL_FISH_VARIABLES
-from pupgui2.datastructures import SteamApp, AWACYStatus, BasicCompatTool, CTType
+from pupgui2.constants import STEAM_STL_INSTALL_PATH, STEAM_STL_CONFIG_PATH, STEAM_STL_SHELL_FILES, STEAM_STL_FISH_VARIABLES, HOME_DIR, IS_FLATPAK
+from pupgui2.datastructures import SteamApp, AWACYStatus, BasicCompatTool, CTType, SteamUser, RuntimeType
 
 
 _cached_app_list = []
 _cached_steam_ctool_id_map = None
 
 
+def get_steam_vdf_compat_tool_mapping(vdf_file: dict) -> dict:
+
+    s = vdf_file.get('InstallConfigStore', {}).get('Software', {})
+
+    # Sometimes the key is 'Valve', sometimes 'valve', see #226
+    c = s.get('Valve') or s.get('valve')
+    if not c:
+        print('Error! config.vdf InstallConfigStore.Software neither contains key "Valve" nor "valve" - config.vdf file may be invalid!')
+        return {}
+
+    m = c.get('Steam', {}).get('CompatToolMapping', {})
+
+    if not m:  # equal to m == {} , may occur after fresh Steam installation
+        print('Warning: CompatToolMapping is empty')
+
+    return m
+
+
 def get_steam_app_list(steam_config_folder: str, cached=False, no_shortcuts=False) -> List[SteamApp]:
     """
     Returns a list of installed Steam apps and optionally game names and the compatibility tool they are using
@@ -37,11 +57,11 @@ def get_steam_app_list(steam_config_folder: str, cached=False, no_shortcuts=Fals
     config_vdf_file = os.path.join(os.path.expanduser(steam_config_folder), 'config.vdf')
 
     apps = []
-    
+
     try:
-        v = vdf.load(open(libraryfolders_vdf_file))
-        c = vdf.load(open(config_vdf_file)).get('InstallConfigStore').get('Software').get('Valve').get('Steam').get('CompatToolMapping')
-        
+        v = vdf_safe_load(libraryfolders_vdf_file)
+        c = get_steam_vdf_compat_tool_mapping(vdf_safe_load(config_vdf_file))
+
         for fid in v.get('libraryfolders'):
             if 'apps' not in v.get('libraryfolders').get(fid):
                 continue
@@ -54,7 +74,7 @@ def get_steam_app_list(steam_config_folder: str, cached=False, no_shortcuts=Fals
                 fid_steamapps_path = os.path.join(fid_libraryfolder_path, 'steamapps')  # e.g. /home/gaben/Games/steamapps
                 appmanifest_path = os.path.join(fid_steamapps_path, f'appmanifest_{appid}.acf')
                 if os.path.isfile(appmanifest_path):
-                    appmanifest_install_path = vdf.load(open(appmanifest_path)).get('AppState', {}).get('installdir', None)
+                    appmanifest_install_path = vdf_safe_load(appmanifest_path).get('AppState', {}).get('installdir', None)
                     if not appmanifest_install_path or not os.path.isdir(os.path.join(fid_steamapps_path, 'common', appmanifest_install_path)):
                         continue
 
@@ -62,16 +82,17 @@ def get_steam_app_list(steam_config_folder: str, cached=False, no_shortcuts=Fals
                 app.app_id = int(appid)
                 app.libraryfolder_id = fid
                 app.libraryfolder_path = fid_path
+                app.anticheat_runtimes = { RuntimeType.EAC: False, RuntimeType.BATTLEYE: False }  # Have to initialize as False here for some reason...
                 if ct := c.get(appid):
                     app.compat_tool = ct.get('name')
                 apps.append(app)
         apps = update_steamapp_info(steam_config_folder, apps)
         apps = update_steamapp_awacystatus(apps)
     except Exception as e:
-        print('Error: Could not get a list of all Steam apps:', e)
-
-    if not no_shortcuts:
-        apps.extend(get_steam_shortcuts_list(steam_config_folder, c))
+        print('Error (get_steam_app_list): Could not get a list of all Steam apps:', e)
+    else:
+        if not no_shortcuts:
+            apps.extend(get_steam_shortcuts_list(steam_config_folder, c))
 
     _cached_app_list = apps
     return apps
@@ -91,10 +112,10 @@ def get_steam_shortcuts_list(steam_config_folder: str, compat_tools: dict=None)
 
     try:
         if not compat_tools:
-            compat_tools = vdf.load(open(config_vdf_file)).get('InstallConfigStore').get('Software').get('Valve').get('Steam').get('CompatToolMapping')
+            compat_tools = get_steam_vdf_compat_tool_mapping(vdf_safe_load(config_vdf_file))
 
-        for file in os.listdir(users_folder):
-            user_directory = os.path.join(users_folder,file)
+        for userf in os.listdir(users_folder):
+            user_directory = os.path.join(users_folder, userf)
             if not os.path.isdir(user_directory):
                 continue
 
@@ -114,19 +135,22 @@ def get_steam_shortcuts_list(steam_config_folder: str, compat_tools: dict=None)
                 
                 app.app_id = appid
                 app.shortcut_id = sid
-                app.shortcut_path = svalue.get('StartDir')
-                app.app_type='game'
-                app.game_name = svalue.get('AppName')
+                app.shortcut_startdir = svalue.get('StartDir')
+                app.shortcut_exe = svalue.get('Exe')
+                app.shortcut_icon = svalue.get('icon')
+                app.shortcut_user = userf
+                app.app_type = 'game'
+                app.game_name = svalue.get('AppName') or svalue.get('appname')
                 if ct := compat_tools.get(str(appid)):
                     app.compat_tool = ct.get('name')
                 apps.append(app)
     except Exception as e:
-        print('Error: Could not get a list of Steam shortcut apps:', e)
+        print('Error (get_steam_shortcuts_list): Could not get a list of Steam shortcut apps:', e)
     
     return apps
 
 
-def get_steam_game_list(steam_config_folder: str, compat_tool='', cached=False) -> List[SteamApp]:
+def get_steam_game_list(steam_config_folder: str, compat_tool: Union[BasicCompatTool, None]=None, cached=False) -> List[SteamApp]:
     """
     Returns a list of installed Steam games and which compatibility tools they are using.
     Specify compat_tool to only return games using the specified tool.
@@ -134,7 +158,20 @@ def get_steam_game_list(steam_config_folder: str, compat_tool='', cached=False)
     """
     apps = get_steam_app_list(steam_config_folder, cached=cached)
 
-    return [app for app in apps if app.app_type == 'game' and (compat_tool == '' or app.compat_tool == compat_tool)]
+    return [app for app in apps if app.app_type == 'game' and (compat_tool is None or app.compat_tool == compat_tool.get_internal_name() or ctool_is_runtime_for_app(app, compat_tool))]
+
+
+def ctool_is_runtime_for_app(app: SteamApp, compat_tool: Union[BasicCompatTool, None]):
+    """
+    Check if a compatibility tool name corresponds to a runtime in use by a SteamApp by comparing a hardcoded name against app.anticheat_runtimes
+    Example: Compatibility tool name is 'ProtonEasyAntiCheatRuntime' and the app.anticheat_runtimes has RuntimeType.EAC as True
+    """
+    if not compat_tool or not compat_tool.ct_type == CTType.STEAM_RT:
+        return False
+
+    compat_tool_name = compat_tool.get_internal_name().lower().replace(' ', '')
+    return 'easyanticheatruntime' in compat_tool_name and app.anticheat_runtimes[RuntimeType.EAC] \
+        or 'battleyeruntime' in compat_tool_name and app.anticheat_runtimes[RuntimeType.BATTLEYE]
 
 
 def get_steam_ct_game_map(steam_config_folder: str, compat_tools: List[BasicCompatTool], cached=False) -> Dict[BasicCompatTool, List[SteamApp]]:
@@ -144,7 +181,7 @@ def get_steam_ct_game_map(steam_config_folder: str, compat_tools: List[BasicComp
     Informal Example: { GE-Proton7-43: [GTA V, Cyberpunk 2077], SteamTinkerLaunch: [Vecter, Terraria] }
     Return Type: Dict[BasicCompatTool, List[SteamApp]]
     """
-    map = {}
+    ct_game_map = {}
 
     apps = get_steam_app_list(steam_config_folder, cached=cached)
 
@@ -152,9 +189,9 @@ def get_steam_ct_game_map(steam_config_folder: str, compat_tools: List[BasicComp
 
     for app in apps:
         if app.app_type == 'game' and app.compat_tool in ct_name_object_map:
-            map.setdefault(ct_name_object_map.get(app.compat_tool), []).append(app)
+            ct_game_map.setdefault(ct_name_object_map.get(app.compat_tool), []).append(app)
 
-    return map
+    return ct_game_map
 
 
 def get_steam_ctool_list(steam_config_folder: str, only_proton=False, cached=False) -> List[SteamApp]:
@@ -174,6 +211,19 @@ def get_steam_ctool_list(steam_config_folder: str, only_proton=False, cached=Fal
     return ctools
 
 
+def get_steam_global_ctool_name(steam_config_folder: str) -> str:
+
+    """
+    Return the internal name of the global Steam compatibility tool selected in the Steam Play settings from the Steam Client.
+    Return Type: str
+    """
+
+    config_vdf_file = os.path.join(os.path.expanduser(steam_config_folder), 'config.vdf')
+    d = get_steam_vdf_compat_tool_mapping(vdf_safe_load(config_vdf_file))
+
+    return d.get('0', {}).get('name', '')
+
+
 def get_steam_acruntime_list(steam_config_folder: str, cached=False) -> List[BasicCompatTool]:
     """
     Returns a list of installed Steam Proton anticheat(EAC/BattlEye) Runtimes.
@@ -208,14 +258,14 @@ def _get_steam_ctool_info(steam_config_folder: str) -> Dict[str, Dict[str, str]]
     compat_tools = {}
     try:
         with open(appinfo_file, 'rb') as f:
-            header, apps = parse_appinfo(f)
+            header, apps = parse_appinfo(f, mapper=dict)
             for steam_app in apps:
                 if steam_app.get('appid') == 891390:
                     compat_tools = steam_app.get('data').get('appinfo').get('extended').get('compat_tools')
                     break
     except Exception as e:
         print('Error getting ctool map from appinfo.vdf:', e)
-    finally:
+    else:
         for t in compat_tools:
             ctool_map[compat_tools.get(t).get('appid')] = {'name': t, 'from_oslist': compat_tools.get(t).get('from_oslist')}
 
@@ -223,28 +273,40 @@ def _get_steam_ctool_info(steam_config_folder: str) -> Dict[str, Dict[str, str]]
     return ctool_map
 
 
-def update_steamapp_info(steam_config_folder: str, steamapp_list: List[SteamApp]) -> List[SteamApp]:
+def update_steamapp_info(steam_config_folder: str, steamapp_list: list[SteamApp]) -> list[SteamApp]:
     """
     Get Steam game names and information for provided SteamApps
     Return Type: List[SteamApp]
     """
     appinfo_file = os.path.join(os.path.expanduser(steam_config_folder), '../appcache/appinfo.vdf')
     appinfo_file = os.path.realpath(appinfo_file)
-    sapps = {app.get_app_id_str(): app for app in steamapp_list}
+    sapps: dict[str, SteamApp] = {app.get_app_id_str(): app for app in steamapp_list}
     len_sapps = len(sapps)
     cnt = 0
     try:
-        ctool_map = _get_steam_ctool_info(steam_config_folder)
+        ctool_map: dict[str, dict[str, str]] = _get_steam_ctool_info(steam_config_folder)
         with open(appinfo_file, 'rb') as f:
-            header, apps = parse_appinfo(f)
+            _, apps = parse_appinfo(f, mapper=dict)
             for steam_app in apps:
                 appid_str = str(steam_app.get('appid'))
                 if a := sapps.get(appid_str):
-                    a.game_name = steam_app.get('data', {}).get('appinfo', {}).get('common', {}).get('name', '')
-                    a.deck_compatibility = steam_app.get('data', {}).get('appinfo', {}).get('common', {}).get('steam_deck_compatibility', {})
-                    if a.game_name.startswith('Proton') and a.game_name.endswith('Runtime'):
+                    app_appinfo = steam_app.get('data', {}).get('appinfo', {})
+                    app_appinfo_common = app_appinfo.get('common', {})
+
+                    # Dictionary of Dictionaries with dependency info, primarily Proton anti-cheat runtimes
+                    # Example: {'0': {'src_os': 'windows', 'dest_os': 'linux', 'appid': 1826330, 'comment': 'EAC runtime'}}
+                    app_additional_dependencies = app_appinfo.get('extended', {}).get('additional_dependencies', {})
+
+                    a.game_name = str(app_appinfo_common.get('name', ''))
+                    a.deck_compatibility = app_appinfo_common.get('steam_deck_compatibility', {})
+                    for dep in app_additional_dependencies.values():
+                        a.anticheat_runtimes[RuntimeType.EAC] = dep.get('appid', -1) == PROTON_EAC_RUNTIME_APPID
+                        a.anticheat_runtimes[RuntimeType.BATTLEYE] = dep.get('appid', -1) == PROTON_BATTLEYE_RUNTIME_APPID
+
+                    # Configure app types
+                    if a.app_id in [PROTON_EAC_RUNTIME_APPID, PROTON_BATTLEYE_RUNTIME_APPID]:
                         a.app_type = 'acruntime'
-                    elif 'Steam Linux Runtime' in a.game_name:
+                    elif a.app_id in [STEAMLINUXRUNTIME_APPID, STEAMLINUXRUNTIME_SOLDIER_APPID, STEAMLINUXRUNTIME_SNIPER_APPID]:
                         a.app_type = 'runtime'
                     elif 'Steamworks' in a.game_name:
                         a.app_type = 'steamworks'
@@ -252,6 +314,8 @@ def update_steamapp_info(steam_config_folder: str, steamapp_list: List[SteamApp]
                         ct = ctool_map.get(steam_app.get('appid'))
                         a.ctool_name = ct.get('name')
                         a.ctool_from_oslist = ct.get('from_oslist')
+                    elif a.app_id == PROTON_NEXT_APPID:  # see https://github.com/DavidoTek/ProtonUp-Qt/pull/280
+                        a.app_type = 'useless-proton-next'
                     else:
                         a.app_type = 'game'
                     cnt += 1
@@ -325,8 +389,8 @@ def steam_update_ctool(game: SteamApp, new_ctool=None, steam_config_folder='') -
     game_id = game.app_id
 
     try:
-        d = vdf.load(open(config_vdf_file))
-        c = d.get('InstallConfigStore').get('Software').get('Valve').get('Steam').get('CompatToolMapping')
+        d = vdf_safe_load(config_vdf_file)
+        c = get_steam_vdf_compat_tool_mapping(d)
 
         if str(game_id) in c:
             if new_ctool is None:
@@ -354,8 +418,8 @@ def steam_update_ctools(games: Dict[SteamApp, str], steam_config_folder='') -> b
         return False
 
     try:
-        d = vdf.load(open(config_vdf_file))
-        c = d.get('InstallConfigStore').get('Software').get('Valve').get('Steam').get('CompatToolMapping')
+        d = vdf_safe_load(config_vdf_file)
+        c = get_steam_vdf_compat_tool_mapping(d)
 
         for game, new_ctool in games.items():
             game_id = game.app_id
@@ -409,7 +473,7 @@ def remove_steamtinkerlaunch(compat_folder='', remove_config=True, ctmod_object=
     """
 
     try:
-        os.chdir(os.path.expanduser('~'))
+        os.chdir(HOME_DIR)
 
         # If the Steam Deck/ProtonUp-Qt installation path doesn't exist
         # Adding `prefix` to path to be especially sure the user didn't just make an `stl` folder
@@ -447,7 +511,7 @@ def remove_steamtinkerlaunch(compat_folder='', remove_config=True, ctmod_object=
                 print(f'Error: SteamTinkerLaunch is installed to {stl_symlink_path}, ProtonUp-Qt cannot modify this folder. Folder must be removed manually.')
         elif os.path.exists(STEAM_STL_INSTALL_PATH):
             # Regular Steam Deck/ProtonUp-Qt installation structure
-            if os.path.exists('/.flatpak-info'):
+            if IS_FLATPAK:
                 if os.path.exists(os.path.join(STEAM_STL_INSTALL_PATH, 'prefix')):
                     shutil.rmtree(os.path.join(STEAM_STL_INSTALL_PATH, 'prefix'))
             else:
@@ -463,7 +527,7 @@ def remove_steamtinkerlaunch(compat_folder='', remove_config=True, ctmod_object=
         # Works by getting all the lines in all the hardcoded Shell files that we write out to during installation and
         # and filtering out any line(s) that reference ProtonUp-Qt, then it writes that updated file content back out to the Shell file
         present_shell_files = [
-            os.path.join(os.path.expanduser('~'), f) for f in os.listdir(os.path.expanduser('~')) if os.path.isfile(os.path.join(os.path.expanduser('~'), f)) and f in STEAM_STL_SHELL_FILES
+            os.path.join(HOME_DIR, f) for f in os.listdir(HOME_DIR) if os.path.isfile(os.path.join(HOME_DIR, f)) and f in STEAM_STL_SHELL_FILES
         ]
         if os.path.exists(STEAM_STL_FISH_VARIABLES) or shutil.which('fish'):
             present_shell_files.append(STEAM_STL_FISH_VARIABLES)
@@ -537,7 +601,7 @@ def install_steam_library_shortcut(steam_config_folder: str, remove_shortcut=Fal
             with open(shortcuts_file, 'wb') as f:
                 if not remove_shortcut:
                     run_config = ['', '']
-                    if os.path.exists('/.flatpak-info'):
+                    if IS_FLATPAK:
                         run_config = [f'/usr/bin/flatpak', f'run {APP_ID}']
                     elif exe := subprocess.run(['which', APP_ID], universal_newlines=True, stdout=subprocess.PIPE).stdout.strip():
                         run_config = [exe, '']
@@ -574,3 +638,201 @@ def install_steam_library_shortcut(steam_config_folder: str, remove_shortcut=Fal
         print(f'Error: Could not add {APP_NAME} as Steam shortcut:', e)
 
     return 0
+
+
+def write_steam_shortcuts_list(steam_config_folder: str, shortcuts: List[SteamApp], delete_sids: List[int]) -> None:
+    """
+    Updates the Steam shortcuts.vdf file with the provided shortcuts
+    It will update existing shortcuts and add new ones
+
+    Parameters:
+        steam_config_folder: str
+            Path to the Steam config folder, e.g. '/home/user/.steam/root/config'
+        shortcuts: List[SteamApp]
+            List of shortcuts to add/update
+        delete_sids: List[int]
+            List of shortcut ids to delete
+    """
+    users_folder = os.path.realpath(os.path.join(os.path.expanduser(steam_config_folder), os.pardir, 'userdata'))
+
+    # group shortcuts by user like this: {user1: {sid1: shortcut1, sid2: shortcut2}, user2: {sid3: shortcut3}}
+    shortcuts_by_user: Dict[Dict[SteamApp]] = {}
+    for shortcut in shortcuts:
+        if shortcut.shortcut_user not in shortcuts_by_user:
+            shortcuts_by_user[shortcut.shortcut_user] = {}
+        shortcuts_by_user[shortcut.shortcut_user][shortcut.shortcut_id] = shortcut
+
+    for userf in shortcuts_by_user:
+        shortcuts_file = os.path.join(users_folder, userf, 'config', 'shortcuts.vdf')
+
+        # read shortcuts.vdf
+        shortcuts_vdf = {}
+        with open(shortcuts_file, 'rb') as f:
+            shortcuts_vdf = vdf.binary_load(f)
+        current_shortcuts = shortcuts_vdf.get('shortcuts', {})
+
+        # update/add new shortcuts
+        modified_shorcuts = shortcuts_by_user.get(userf, {})
+        for sid in list(modified_shorcuts.keys()):
+            shortcut_modified: SteamApp = modified_shorcuts.get(sid)
+            if sid in current_shortcuts:  # update existing shortcut
+                svalue_current = current_shortcuts.get(sid)
+                svalue_current['AppName'] = shortcut_modified.game_name
+                svalue_current['Exe'] = shortcut_modified.shortcut_exe
+                svalue_current['StartDir'] = shortcut_modified.shortcut_startdir
+                svalue_current['icon'] = shortcut_modified.shortcut_icon
+            else:  # add a new shortcut to shortcuts.vdf
+                svalue_new = {
+                    'appid': shortcut_modified.app_id,
+                    'AppName': shortcut_modified.game_name,
+                    'Exe': shortcut_modified.shortcut_exe,
+                    'StartDir': shortcut_modified.shortcut_startdir,
+                    'icon': shortcut_modified.shortcut_icon,
+                    'ShortcutPath': '',
+                    'LaunchOptions': '',
+                    'IsHidden': 0,
+                    'AllowDesktopConfig': 1,
+                    'AllowOverlay': 1,
+                    'OpenVR': 0,
+                    'Devkit': 0,
+                    'DevkitGameID': '',
+                    'DevkitOverrideAppID': 0,
+                    'LastPlayTime': 0,
+                    'FlatpakAppID': '',
+                    'tags': {}
+                }
+                current_shortcuts[sid] = svalue_new
+
+        # delete shortcuts that are marked for deletion
+        for sid in delete_sids:
+            current_shortcuts.pop(sid)
+
+        # write shortcuts.vdf
+        try:
+            with open(shortcuts_file, 'wb') as f:
+                f.write(vdf.binary_dumps(shortcuts_vdf))
+        except Exception as e:
+            print(f'Error: Could not write_steam_shortcuts_list for user {userf}:', e)
+
+
+def calc_shortcut_app_id(appname: str, exe: str) -> int:
+    """
+    Calculates an app id for a shortcut based on the app name and executable.
+    Based on https://github.com/SteamGridDB/steam-rom-manager/blob/master/src/lib/helpers/steam/generate-app-id.ts
+
+    Parameters:
+        appname: str
+            game_name of the shortcut
+        exe: str
+            shortcut_exe
+
+    Returns:
+        int
+    """
+    key = exe + appname
+    return (binascii.crc32(key.encode()) | 0x80000000) - 0x100000000
+
+
+def get_steam_user_list(steam_config_folder: str) -> List[SteamUser]:
+    """
+    Returns a list of Steam users
+
+    Parameters:
+        steam_config_folder: str
+            e.g. '~/.steam/root/config'
+
+    Return Type: List[SteamUser]
+    """
+    loginusers_vdf_file = os.path.join(os.path.expanduser(steam_config_folder), 'loginusers.vdf')
+
+    users = []
+
+    if not os.path.exists(loginusers_vdf_file):
+        print(f'Warning: Loginusers file does not exist at {loginusers_vdf_file}')
+        return []
+
+    try:
+        d = vdf_safe_load(loginusers_vdf_file)
+        u = d.get('users', {})
+        for uid in list(u.keys()):
+            uvalue = u.get(uid, {})
+
+            user = SteamUser()
+            user.long_id = int(uid)
+            user.account_name = uvalue.get('AccountName', '')
+            user.persona_name = uvalue.get('PersonaName', '')
+            user.most_recent = bool(int(uvalue.get('MostRecent', '0')))
+            user.timestamp = int(uvalue.get('Timestamp', '-1'))
+
+            users.append(user)
+    except Exception as e:
+        print('Error (get_steam_user_list): Could not get a list of Steam users:', e)
+
+    return users
+
+
+def determine_most_recent_steam_user(steam_users: List[SteamUser]) -> SteamUser:
+    """
+    Returns the Steam user that was logged-in most recent, otherwise the first user or None.
+    Looks for the first user with most_recent=True.
+
+    Parameters:
+        steam_users: List[SteamUser]
+            list of steam users, from get_steam_user_list()
+
+    Return Type: SteamUser|None
+    """
+    for user in steam_users:
+        if user.most_recent == True:
+            return user
+
+    if len(steam_users) > 0:
+        print(f'Warning: There is no most recent Steam user. Returning the first user {steam_users[0].get_short_id()}')
+        return steam_users[0]
+
+    print('Warning: No Steam users found. Returning None')
+    return None
+
+
+def is_valid_steam_install(steam_path) -> bool:
+
+    """
+    Return whether required Steam data files actually exist to determine if 'steam_path' is a valid Steam installation.
+    Return Type: bool
+    """
+
+    ct_dir = os.path.join(os.path.expanduser(steam_path), 'config')
+
+    config_vdf = os.path.join(ct_dir, 'config.vdf')
+    libraryfolders_vdf = os.path.join(ct_dir, 'libraryfolders.vdf')
+
+    is_valid_steam_install = os.path.exists(config_vdf) and os.path.exists(libraryfolders_vdf)
+
+    return is_valid_steam_install
+
+def vdf_safe_load(vdf_file: str) -> dict:
+    """
+    Loads a vdf file and returns its contents as a dict.
+    In case of an error, the error is printed and {} is returned.
+
+    Args:
+        vdf_file (str): Path to the vdf file
+
+    Returns:
+        dict (empty in case of an error)
+    """
+    data = {}
+
+    try:
+        # See https://github.com/DavidoTek/ProtonUp-Qt/issues/424 (unicode errors)
+        with open(vdf_file, 'r', encoding='utf-8', errors='replace') as f:
+            data = vdf.loads(f.read())
+    except Exception as e:
+        print(f'An error occured while calling vdf_safe_load("{vdf_file}"). Returning empty dict: {e}')
+
+    if not isinstance(data, dict):
+        # Apparently, vdf.loads() can return None (issue #481)
+        print(f'Warning (vdf_safe_load): vdf.loads("{vdf_file}") returned {data}. Returning empty dict.')
+        data = {}
+
+    return data
